<?php
namespace Pinecast;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Podcast {

	/** @var int ID of the podcast. */
	private $id = null;

	/** @var string Name of the podcast. */
	private $name = null;

	/** @var string Podcast feed URL. */
	private $feed_url = null;

	/** @var int[] Category IDs */
	private $categories = array();

	/** @var bool Featured image usage */
	private $featured_image = true;

	/** @var string Last time the podcast was synced. */
	private $last_updated = null;


	/**
	 * Constructor.
	 */
	public function __construct() {}


	/**
	 * Get feed ID.
	 *
	 * @return int Feed ID.
	 */
	public function get_id() {
		return $this->id;
	}


	/**
	 * Set podcast ID.
	 *
	 * @param int $id Podcast ID.
	 * @return Podcast $this Current object.
	 */
	private function set_id( $id ) {
		$this->id = absint( $id );

		return $this;
	}


	/**
	 * Setup podcast.
	 */
	public static function create( $id, $data ) {
		$podcast = new Podcast;

		$podcast
			->set_id( $id )
			->set_name( $data['name'] ?? '' )
			->set_feed_url( $data['feed_url'] ?? '' )
			->set_category_ids( $data['categories'] ?? array() )
			->set_featured_image( $data['featured_image'] ?? true )
			->set_last_updated( $data['last_updated'] ?? '' );

		return $podcast;
	}


	/**
	 * Save changes made.
	 *
	 * @return bool True on success, false otherwise.
	 */
	public function update() {
		$podcasts = get_option( 'pinecast_feeds', array() );

		$existing = $podcasts[ $this->get_id() ];
		$podcasts[ $this->get_id() ] = array(
			'id'             => $this->get_id(),
			'name'           => $this->get_name(),
			'feed_url'       => $existing ? $existing['feed_url'] : $this->get_feed_url(),
			'categories'     => $this->get_category_ids(),
			'featured_image' => $this->use_featured_image(),
			'last_updated'   => $this->get_last_updated(),
		);

		update_option( 'pinecast_feeds', $podcasts );

		return true;
	}


	/**
	 * Delete podcast.
	 *
	 * @return bool True on success, false otherwise.
	 */
	public function delete() {
		$podcasts = get_option( 'pinecast_feeds', array() );

		if ( isset( $podcasts[ $this->get_id() ] ) ) {
			unset( $podcasts[ $this->get_id() ] );
		}

		update_option( 'pinecast_feeds', $podcasts );

		global $wpdb;
		$results = $wpdb->get_results(
			$wpdb->prepare(
				"SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = '_pinecast_feed' AND meta_value = %s",
				$this->feed_url
			),
			OBJECT_K
		);
		$existing_episode_ids = array_keys( $results );
		foreach ($existing_episode_ids as $id) {
			wp_delete_post($id);
		}

		return true;
	}


	/**
	 * Get podcast name.
	 *
	 * @return string Podcast name.
	 */
	public function get_name() {
		return $this->name;
	}


	/**
	 * Set podcast name.
	 *
	 * @param string $name Podcast name.
	 * @return Podcast $this Current object.
	 */
	public function set_name( $name ) {
		$this->name = sanitize_text_field( $name );

		return $this;
	}

	/**
	 * Get feed URL.
	 *
	 * @return string The feed URL.
	 */
	public function get_feed_url() {
		$url = str_replace( 'https://pinecast.com/jsonfeed/', 'https://pinecast.com/feed/', $this->feed_url );

		return $url;
	}


	/**
	 * Set feed URL.
	 *
	 * @param string $url Feed url.
	 * @return Podcast $this Current object.
	 */
	public function set_feed_url( $url ) {
		$url = preg_replace(
			'/^https:\\/\\/pinecast\\.com\\/feed\\//i',
			'https://pinecast.com/jsonfeed/',
			$url
		);
		$this->feed_url = esc_url_raw( $url );

		return $this;
	}


	/**
	 * Get the actual feed.
	 *
	 * Get the feed from Pinecast with the podcast episodes.
	 */
	public function get_feed() {
		$response = wp_remote_get(
			$this->feed_url,
			['user-agent' => 'WordPress']
		);
		$body     = wp_remote_retrieve_body( $response );
		$body     = json_decode( $body );

		return $body;
	}


	/**
	 * Get last updated.
	 *
	 * @return string Time the podcast was last updated/synced.
	 */
	public function get_last_updated() {
		return $this->last_updated;
	}


	/**
	 * Update last updated.
	 *
	 * @param string|null $time The time the podcast was last updated on the site. Omit to use current time.
	 * @return Podcast Current object.
	 */
	public function set_last_updated( $time = null ) {
		if ( $time === null ) {
			$time = time();
		}

		if ( ! empty( $time ) ) {
			$this->last_updated = date_i18n( 'Y-m-d H:i:s', strtotime( $time ) );
		}

		return $this;
	}


	/**
	 * Get category IDs.
	 *
	 * Get the category ID(s) to assign to this podcast's episodes.
	 *
	 * @return int[] The category IDs.
	 */
	public function get_category_ids() {
		return $this->categories;
	}


	/**
	 * Set category IDs.
	 *
	 * @param int[] $ids Category IDs to set.
	 * @return Podcast $this Current object.
	 */
	public function set_category_ids( $ids = array() ) {
		$this->categories = array_map( 'intval', $ids );

		return $this;
	}


	/**
	 * Get featured image usage.
	 *
	 * Get whether or not the featured image should be set to the episode image.
	 *
	 * @return mixed
	 */
	public function use_featured_image() {
		return $this->featured_image;
	}


	/**
	 * Set featured image usage.
	 *
	 * @param bool $value Whether or not to use episode images as featured image.
	 * @return Podcast $this Current object.
	 */
	public function set_featured_image( $value ) {
		$this->featured_image = (bool) $value;

		return $this;
	}


	/**
	 * Check if episodes can be synced.
	 *
	 * Checks if the feed URL is entered and valid. Does NOT check
	 * validity of feed itself.
	 *
	 * @return bool Whether the episodes can be synced.
	 */
	public function can_sync() {
		return ! empty( $this->get_feed_url() );
	}


	/**
	 * Synchronize episodes.
	 *
	 * Synchronize the podcast episodes, create new posts if they don't exist yet.
	 *
	 * @return int[]|bool List of post IDs created on success (can be empty), false on failure.
	 */
	public function sync_episodes() {
		$feed = $this->get_feed();

		if ( empty( $feed->items ) ) {
			return false;
		}

		global $wpdb;
		$results = $wpdb->get_results( "SELECT meta_value, post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = '_pinecast_id'", OBJECT_K );
		$existing_episode_ids = array_keys( $results );

		$created_posts = array();
		foreach ( $feed->items as $episode ) {
			if ( in_array( $episode->id, $existing_episode_ids ) ) {
				$this->update_episode_post( $episode, $results[$episode->id]->post_id );
			} elseif ( $post_id = $this->create_episode_post( $episode ) ) {
				$created_posts[] = $post_id;
			}
		}

		return $created_posts;
	}


	/**
	 * Gets the data for a post
	 *
	 * @param array $data Data provided through the RSS feed.
	 * @return array Blob of post data for wp_insert_post or wp_update_post
	 */
	private function get_post_data( $data ) {
		$audio_attachment = reset( $data->attachments );
		$audio_url = remove_query_arg( array( 'source', 'ext' ), $audio_attachment->url ) ?? '';

		$post_data = apply_filters( 'pinecast/create_episode_args', array(
			'post_title'    => sanitize_text_field( $data->title ),
			'post_content'  => '[audio src="' . $audio_url . '"]' . '<br>' . wp_kses_post( $data->content_html ),
			'post_status'   => 'publish',
			'post_date'     => date_i18n( 'Y-m-d H:i:s', strtotime( $data->date_published ) ),
			'post_category' => $this->get_category_ids(),
			'meta_input'    => array(
				'_pinecast_id' => $data->id,
				'_pinecast_image' => $data->image,
				'_pinecast_feed' => $this->feed_url,
			),
		), $this, $data );

		return $post_data;
	}


	/**
	 * Create episode post.
	 *
	 * Create a new post for a episode.
	 *
	 * @param array $data Data provided through the RSS feed.
	 * @return bool|int Created post ID. False on failure.
	 */
	private function create_episode_post( $data ) {
		$post_data = $this->get_post_data($data);
		$post_id = wp_insert_post( $post_data );

		if ( is_wp_error( $post_id ) ) {
			return false;
		}

		if ( $this->use_featured_image() && ! empty( $data->image ) ) {
			if ( $attachment_id = $this->import_episode_image( $data->image, $post_id ) ) {
				set_post_thumbnail( $post_id, $attachment_id );
			}
		}

		do_action( 'pinecast/create_episode', $post_id, $data );

		return $post_id;
	}


	/**
	 * Update episode post.
	 *
	 * @param array $data Data provided through the RSS feed.
	 * @param int $id The ID of the existing post
	 * @return bool|int Created post ID. False on failure.
	 */
	private function update_episode_post( $data, $id ) {
		$post_data = $this->get_post_data($data);
		$post_data['ID'] = $id;
		$post_id = wp_insert_post( $post_data );

		if ( is_wp_error( $post_id ) ) {
			return false;
		}

		if ( $this->use_featured_image() ) {
			if ($data->image && $attachment_id = $this->import_episode_image( $data->image, $post_id ) ) {
				set_post_thumbnail( $post_id, $attachment_id );
			}
		} else {
			if (has_post_thumbnail($post_id)) {
				delete_post_thumbnail($post_id);
				delete_metadata('post', $post_id, '_pinecast_image');
			}
		}

		do_action( 'pinecast/update_episode', $post_id, $data );

		return $post_id;
	}


	/**
	 * Import episode image.
	 *
	 * Import the featured image to the post of a episode.
	 *
	 * @param string $image_url Image URL to import.
	 * @param int    $post_id ID of the post related to the episode.
	 * @return bool|int Attachment ID on success, false otherwise.
	 */
	private function import_episode_image( $image_url, $post_id ) {
		require_once ABSPATH . 'wp-includes/post.php';
		require_once ABSPATH . 'wp-admin/includes/image.php';
		require_once ABSPATH . 'wp-admin/includes/file.php';
		require_once ABSPATH . 'wp-admin/includes/media.php';

		// Download file to temp dir
		$temp_file = download_url( $image_url, $timeout_seconds = 5 );

		if ( ! is_wp_error( $temp_file ) ) {
			$file = array(
				'name'     => basename( $image_url ),
				'type'     => mime_content_type( $temp_file ),
				'tmp_name' => $temp_file,
				'error'    => 0,
				'size'     => filesize( $temp_file ),
			);

			// Move temporary file to uploads directory
			$attachment_id = media_handle_sideload( $file, $post_id );

			return $attachment_id;
		}

		return false;
	}
}
